<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

/**
 * DLM_Admin class.
 */
class DLM_Admin {

	private $settings;

	/**
	 * Setup actions etc.
	 */
	public function setup() {
		// Directory protection
		add_filter( 'mod_rewrite_rules', array( $this, 'ms_files_protection' ) );
		add_filter( 'upload_dir', array( $this, 'upload_dir' ) );

		add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) );
		add_action( 'admin_menu', array( $this, 'admin_menu' ), 12 );
		add_action( 'admin_init', array( $this, 'register_settings' ) );
		add_action( 'admin_init', array( $this, 'export_logs' ) );
		add_action( 'admin_init', array( $this, 'delete_logs' ) );
		add_action( 'wp_dashboard_setup', array( $this, 'admin_dashboard' ) );
	}

	/**
	 * ms_files_protection function.
	 *
	 * @access public
	 *
	 * @param mixed $rewrite
	 *
	 * @return void
	 */
	public function ms_files_protection( $rewrite ) {

		if ( ! is_multisite() ) {
			return $rewrite;
		}

		$rule = "\n# DLM Rules - Protect Files from ms-files.php\n\n";
		$rule .= "<IfModule mod_rewrite.c>\n";
		$rule .= "RewriteEngine On\n";
		$rule .= "RewriteCond %{QUERY_STRING} file=dlm_uploads/ [NC]\n";
		$rule .= "RewriteRule /ms-files.php$ - [F]\n";
		$rule .= "</IfModule>\n\n";

		return $rule . $rewrite;
	}

	/**
	 * upload_dir function.
	 *
	 * @access public
	 *
	 * @param mixed $pathdata
	 *
	 * @return void
	 */
	public function upload_dir( $pathdata ) {

		if ( isset( $_POST['type'] ) && 'dlm_download' === $_POST['type'] ) {
			if ( empty( $pathdata['subdir'] ) ) {
				$pathdata['path']   = $pathdata['path'] . '/dlm_uploads';
				$pathdata['url']    = $pathdata['url'] . '/dlm_uploads';
				$pathdata['subdir'] = '/dlm_uploads';
			} else {
				$new_subdir = '/dlm_uploads' . $pathdata['subdir'];

				$pathdata['path']   = str_replace( $pathdata['subdir'], $new_subdir, $pathdata['path'] );
				$pathdata['url']    = str_replace( $pathdata['subdir'], $new_subdir, $pathdata['url'] );
				$pathdata['subdir'] = str_replace( $pathdata['subdir'], $new_subdir, $pathdata['subdir'] );
			}
		}

		return $pathdata;
	}

	/**
	 * init_settings function.
	 *
	 * @access private
	 * @return void
	 */
	private function init_settings() {
		$this->settings = apply_filters( 'download_monitor_settings',
			array(
				'general'   => array(
					__( 'General', 'download-monitor' ),
					array(
						array(
							'name'    => 'dlm_default_template',
							'std'     => '',
							'label'   => __( 'Default Template', 'download-monitor' ),
							'desc'    => __( 'Choose which template is used for <code>[download]</code> shortcodes by default (this can be overridden by the <code>format</code> argument).', 'download-monitor' ),
							'type'    => 'select',
							'options' => array(
								''             => __( 'Default - Title and count', 'download-monitor' ),
								'button'       => __( 'Button - CSS styled button showing title and count', 'download-monitor' ),
								'box'          => __( 'Box - Box showing thumbnail, title, count, filename and filesize.', 'download-monitor' ),
								'filename'     => __( 'Filename - Filename and download count', 'download-monitor' ),
								'title'        => __( 'Title - Shows download title only', 'download-monitor' ),
								'version-list' => __( 'Version list - Lists all download versions in an unordered list', 'download-monitor' ),
								'custom'       => __( 'Custom template', 'download-monitor' )
							)
						),
						array(
							'name'  => 'dlm_custom_template',
							'std'   => '',
							'label' => __( 'Custom Template', 'download-monitor' ),
							'desc'  => __( 'Leaving this blank will use the default <code>content-download.php</code> template file. If you enter, for example, <code>image</code>, the <code>content-download-image.php</code> template will be used instead. You can add custom templates inside your theme folder.', 'download-monitor' )
						),
						array(
							'name'     => 'dlm_generate_hash_md5',
							'std'      => '0',
							'label'    => __( 'MD5 hashes', 'download-monitor' ),
							'cb_label' => __( 'Generate MD5 hash for uploaded files', 'download-monitor' ),
							'desc'     => '',
							'type'     => 'checkbox'
						),
						array(
							'name'     => 'dlm_generate_hash_sha1',
							'std'      => '0',
							'label'    => __( 'SHA1 hashes', 'download-monitor' ),
							'cb_label' => __( 'Generate SHA1 hash for uploaded files', 'download-monitor' ),
							'desc'     => '',
							'type'     => 'checkbox'
						),
						array(
							'name'     => 'dlm_generate_hash_crc32b',
							'std'      => '0',
							'label'    => __( 'CRC32B hashes', 'download-monitor' ),
							'cb_label' => __( 'Generate CRC32B hash for uploaded files', 'download-monitor' ),
							'desc'     => __( 'Hashes can optionally be output via shortcodes, but may cause performance issues with large files.', 'download-monitor' ),
							'type'     => 'checkbox'
						),
					),
				),
				'endpoints' => array(
					__( 'Endpoint', 'download-monitor' ),
					array(
						array(
							'name'        => 'dlm_download_endpoint',
							'std'         => 'download',
							'placeholder' => __( 'download', 'download-monitor' ),
							'label'       => __( 'Download Endpoint', 'download-monitor' ),
							'desc'        => sprintf( __( 'Define what endpoint should be used for download links. By default this will be <code>%s</code>.', 'download-monitor' ), home_url( '/download/' ) )
						),
						array(
							'name'    => 'dlm_download_endpoint_value',
							'std'     => 'ID',
							'label'   => __( 'Endpoint Value', 'download-monitor' ),
							'desc'    => sprintf( __( 'Define what unique value should be used on the end of your endpoint to identify the downloadable file. e.g. ID would give a link like <code>%s</code>', 'download-monitor' ), home_url( '/download/10/' ) ),
							'type'    => 'select',
							'options' => array(
								'ID'   => __( 'Download ID', 'download-monitor' ),
								'slug' => __( 'Download slug', 'download-monitor' )
							)
						),
						array(
							'name'     => 'dlm_xsendfile_enabled',
							'std'      => '',
							'label'    => __( 'X-Accel-Redirect / X-Sendfile', 'download-monitor' ),
							'cb_label' => __( 'Enable', 'download-monitor' ),
							'desc'     => __( 'If supported, <code>X-Accel-Redirect</code> / <code>X-Sendfile</code> can be used to serve downloads instead of PHP (server requires <code>mod_xsendfile</code>).', 'download-monitor' ),
							'type'     => 'checkbox'
						),
						array(
							'name'     => 'dlm_hotlink_protection_enabled',
							'std'      => '',
							'label'    => __( 'Prevent hotlinking', 'download-monitor' ),
							'cb_label' => __( 'Enable', 'download-monitor' ),
							'desc'     => __( 'If enabled, the download handler will check the PHP referer to see if it originated from your site and if not, redirect them to the homepage.', 'download-monitor' ),
							'type'     => 'checkbox'
						)
					)
				),
				'logging'   => array(
					__( 'Logging', 'download-monitor' ),
					array(
						array(
							'name'     => 'dlm_enable_logging',
							'cb_label' => __( 'Enable', 'download-monitor' ),
							'std'      => '1',
							'label'    => __( 'Download Log', 'download-monitor' ),
							'desc'     => __( 'Log download attempts, IP addresses and more.', 'download-monitor' ),
							'type'     => 'checkbox'
						),
						array(
							'name'        => 'dlm_ip_blacklist',
							'std'         => '192.168.0.*',
							'label'       => __( 'Blacklist IPs', 'download-monitor' ),
							'desc'        => __( 'List IP Addresses to blacklist, 1 per line. Use <code>*</code> for a wildcard.', 'download-monitor' ),
							'placeholder' => '',
							'type'        => 'textarea'
						),
						array(
							'name'        => 'dlm_user_agent_blacklist',
							'std'         => 'Googlebot',
							'label'       => __( 'Blacklist user agents', 'download-monitor' ),
							'desc'        => __( 'List browser user agents to blacklist, 1 per line.', 'download-monitor' ),
							'placeholder' => '',
							'type'        => 'textarea'
						),
					)
				)
			)
		);
	}

	/**
	 * register_settings function.
	 *
	 * @access public
	 * @return void
	 */
	public function register_settings() {
		$this->init_settings();

		foreach ( $this->settings as $section ) {
			foreach ( $section[1] as $option ) {
				if ( isset( $option['std'] ) ) {
					add_option( $option['name'], $option['std'] );
				}
				register_setting( 'download-monitor', $option['name'] );
			}
		}
	}

	/**
	 * admin_enqueue_scripts function.
	 *
	 * @access public
	 * @return void
	 */
	public function admin_enqueue_scripts( $hook ) {
		global $post;

		wp_enqueue_style( 'download_monitor_menu_css', WP_DLM::get_plugin_url() . '/assets/css/menu.css' );

		if ( $hook == 'index.php' ) {
			wp_enqueue_style( 'download_monitor_dashboard_css', WP_DLM::get_plugin_url() . '/assets/css/dashboard.css' );
		}

		$enqueue = false;

		if ( $hook == 'post-new.php' || $hook == 'post.php' || $hook == 'edit.php' ) {
			if ( ( ! empty( $_GET['post_type'] ) && $_GET['post_type'] == 'dlm_download' ) || ( ! empty( $post->post_type ) && 'dlm_download' === $post->post_type ) ) {
				$enqueue = true;
			}
		}

		if ( strstr( $hook, 'dlm_download_page' ) ) {
			$enqueue = true;
		}

		if ( $hook == 'edit-tags.php' && strstr( $_GET['taxonomy'], 'dlm_download' ) ) {
			$enqueue = true;
		}

		if ( ! $enqueue ) {
			return;
		}

		wp_enqueue_script( 'jquery-blockui', WP_DLM::get_plugin_url() . '/assets/js/blockui.min.js', array( 'jquery' ), '2.61' );
		wp_enqueue_script( 'jquery-ui-sortable' );
		wp_enqueue_script( 'jquery-ui-datepicker' );
		wp_enqueue_style( 'jquery-ui-style', ( is_ssl() ) ? 'https://ajax.googleapis.com/ajax/libs/jqueryui/1.8.2/themes/smoothness/jquery-ui.css' : 'http://ajax.googleapis.com/ajax/libs/jqueryui/1.8.2/themes/smoothness/jquery-ui.css' );
		wp_enqueue_style( 'download_monitor_admin_css', WP_DLM::get_plugin_url() . '/assets/css/admin.css', array( 'dashicons' ) );
	}

	/**
	 * admin_menu function.
	 *
	 * @access public
	 * @return void
	 */
	public function admin_menu() {

		// Logging object
		$logging = new DLM_Logging();

		// Logs page
		if ( $logging->is_logging_enabled() ) {
			add_submenu_page( 'edit.php?post_type=dlm_download', __( 'Logs', 'download-monitor' ), __( 'Logs', 'download-monitor' ), 'manage_options', 'download-monitor-logs', array(
				$this,
				'log_viewer'
			) );
		}

		// Settings page
		add_submenu_page( 'edit.php?post_type=dlm_download', __( 'Settings', 'download-monitor' ), __( 'Settings', 'download-monitor' ), 'manage_options', 'download-monitor-settings', array(
				$this,
				'settings_page'
			) );

		// Extensions page
		add_submenu_page( 'edit.php?post_type=dlm_download', __( 'Download Monitor Extensions', 'download-monitor' ), __( 'Extensions', 'download-monitor' ), 'manage_options', 'dlm-extensions', array(
			$this,
			'extensions_page'
		) );

	}

	/**
	 * Output extensions page
	 */
	public function extensions_page() {
		$admin_extensions = new DLM_Admin_Extensions();
		$admin_extensions->output();
	}

	/**
	 * settings_page function.
	 *
	 * @access public
	 * @return void
	 */
	public function settings_page() {
		$this->init_settings();
		?>
		<div class="wrap">
			<form method="post" action="options.php">

				<?php settings_fields( 'download-monitor' ); ?>
				<?php screen_icon(); ?>

				<h2 class="nav-tab-wrapper">
					<?php
					foreach ( $this->settings as $key => $section ) {
						echo '<a href="#settings-' . sanitize_title( $key ) . '" class="nav-tab">' . esc_html( $section[0] ) . '</a>';
					}
					?>
				</h2><br/>

				<?php
				if ( ! empty( $_GET['settings-updated'] ) ) {
					flush_rewrite_rules();
					echo '<div class="updated fade"><p>' . __( 'Settings successfully saved', 'download-monitor' ) . '</p></div>';
				}

				foreach ( $this->settings as $key => $section ) {

					echo '<div id="settings-' . sanitize_title( $key ) . '" class="settings_panel">';

					echo '<table class="form-table">';

					foreach ( $section[1] as $option ) {

						$placeholder = ( ! empty( $option['placeholder'] ) ) ? 'placeholder="' . $option['placeholder'] . '"' : '';

						echo '<tr valign="top"><th scope="row"><label for="setting-' . $option['name'] . '">' . $option['label'] . '</a></th><td>';

						if ( ! isset( $option['type'] ) ) {
							$option['type'] = '';
						}

						$value = get_option( $option['name'] );

						switch ( $option['type'] ) {

							case "checkbox" :

								?><label><input id="setting-<?php echo $option['name']; ?>"
								                name="<?php echo $option['name']; ?>" type="checkbox"
								                value="1" <?php checked( '1', $value ); ?> /> <?php echo $option['cb_label']; ?>
								</label><?php

								if ( $option['desc'] ) {
									echo ' <p class="description">' . $option['desc'] . '</p>';
								}

								break;
							case "textarea" :

								?><textarea id="setting-<?php echo $option['name']; ?>" class="large-text" cols="50"
								            rows="3"
								            name="<?php echo $option['name']; ?>" <?php echo $placeholder; ?>><?php echo esc_textarea( $value ); ?></textarea><?php

								if ( $option['desc'] ) {
									echo ' <p class="description">' . $option['desc'] . '</p>';
								}

								break;
							case "select" :

								?><select id="setting-<?php echo $option['name']; ?>" class="regular-text"
								          name="<?php echo $option['name']; ?>"><?php
								foreach ( $option['options'] as $key => $name ) {
									echo '<option value="' . esc_attr( $key ) . '" ' . selected( $value, $key, false ) . '>' . esc_html( $name ) . '</option>';
								}
								?></select><?php

								if ( $option['desc'] ) {
									echo ' <p class="description">' . $option['desc'] . '</p>';
								}

								break;
							default :

								?><input id="setting-<?php echo $option['name']; ?>" class="regular-text" type="text"
								         name="<?php echo $option['name']; ?>"
								         value="<?php esc_attr_e( $value ); ?>" <?php echo $placeholder; ?> /><?php

								if ( $option['desc'] ) {
									echo ' <p class="description">' . $option['desc'] . '</p>';
								}

								break;

						}

						echo '</td></tr>';
					}

					echo '</table></div>';

				}
				?>
				<p class="submit">
					<input type="submit" class="button-primary"
					       value="<?php _e( 'Save Changes', 'download-monitor' ); ?>"/>
				</p>
			</form>
		</div>
		<?php
	}

	/**
	 * log_viewer function.
	 *
	 * @access public
	 * @return void
	 */
	function log_viewer() {
		if ( ! class_exists( 'WP_List_Table' ) ) {
			require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
		}

		require_once( 'class-dlm-logging-list-table.php' );

		$DLM_Logging_List_Table = new DLM_Logging_List_Table();
		$DLM_Logging_List_Table->prepare_items();
		?>
		<div class="wrap">
			<div id="icon-edit" class="icon32 icon32-posts-dlm_download"><br/></div>

			<h2><?php _e( 'Download Logs', 'download-monitor' ); ?> <a
					href="<?php echo add_query_arg( 'dlm_download_logs', 'true' ); ?>"
					class="add-new-h2"><?php _e( 'Export CSV', 'download-monitor' ); ?></a> <a
					href="<?php echo wp_nonce_url( add_query_arg( 'dlm_delete_logs', 'true' ), 'delete_logs' ); ?>"
					class="add-new-h2"><?php _e( 'Delete Logs', 'download-monitor' ); ?></a></h2><br/>

			<form id="dlm_logs">
				<?php $DLM_Logging_List_Table->display() ?>
			</form>
		</div>
	<?php
	}

	/**
	 * Delete logs
	 */
	public function delete_logs() {
		global $wpdb;

		if ( empty( $_GET['dlm_delete_logs'] ) ) {
			return;
		}

		check_admin_referer( 'delete_logs' );

		$wpdb->query( "DELETE FROM {$wpdb->download_log};" );
	}

	/**
	 * export_logs function
	 */
	public function export_logs() {
		global $wpdb;

		if ( empty( $_GET['dlm_download_logs'] ) ) {
			return;
		}

		$filter_status = isset( $_REQUEST['filter_status'] ) ? sanitize_text_field( $_REQUEST['filter_status'] ) : '';
		$filter_month  = ! empty( $_REQUEST['filter_month'] ) ? sanitize_text_field( $_REQUEST['filter_month'] ) : '';

		$items = $wpdb->get_results(
			$wpdb->prepare(
				"SELECT * FROM {$wpdb->download_log}
		    	WHERE type = 'download'
		    	" . ( $filter_status ? "AND download_status = '%s'" : "%s" ) . "
	            " . ( $filter_month ? "AND download_date >= '%s'" : "%s" ) . "
	            " . ( $filter_month ? "AND download_date <= '%s'" : "%s" ) . "
		    	ORDER BY download_date DESC",
				( $filter_status ? $filter_status : "" ),
				( $filter_month ? date( 'Y-m-01', strtotime( $filter_month ) ) : "" ),
				( $filter_month ? date( 'Y-m-t', strtotime( $filter_month ) ) : "" )
			)
		);

		$rows   = array();
		$row    = array();
		$row[]  = __( 'Download ID', 'download-monitor' );
		$row[]  = __( 'Version ID', 'download-monitor' );
		$row[]  = __( 'Filename', 'download-monitor' );
		$row[]  = __( 'User ID', 'download-monitor' );
		$row[]  = __( 'User Login', 'download-monitor' );
		$row[]  = __( 'User Email', 'download-monitor' );
		$row[]  = __( 'User IP', 'download-monitor' );
		$row[]  = __( 'User Agent', 'download-monitor' );
		$row[]  = __( 'Date', 'download-monitor' );
		$row[]  = __( 'Status', 'download-monitor' );
		$rows[] = '"' . implode( '","', $row ) . '"';

		if ( ! empty( $items ) ) {
			foreach ( $items as $item ) {
				$row   = array();
				$row[] = $item->download_id;
				$row[] = $item->version_id;

				$download = new DLM_Download( $item->download_id );
				$download->set_version( $item->version_id );

				if ( $download->exists() && $download->get_the_filename() ) {
					$row[] = $download->get_the_filename();
				} else {
					$row[] = '-';
				}

				$row[] = $item->user_id;

				if ( $item->user_id ) {
					$user = get_user_by( 'id', $item->user_id );
				}

				if ( ! isset( $user ) || ! $user ) {
					$row[] = '-';
					$row[] = '-';
				} else {
					$row[] = $user->user_login;
					$row[] = $user->user_email;
				}

				$row[]  = $item->user_ip;
				$row[]  = $item->user_agent;
				$row[]  = $item->download_date;
				$row[]  = $item->download_status . ( $item->download_status_message ? ' - ' : '' ) . $item->download_status_message;
				$rows[] = '"' . implode( '","', $row ) . '"';
			}
		}

		$log = implode( "\n", $rows );

		header( "Content-type: text/csv" );
		header( "Content-Disposition: attachment; filename=download_log.csv" );
		header( "Cache-Control: must-revalidate, post-check=0, pre-check=0" );
		header( "Content-Length: " . strlen( $log ) );
		echo $log;
		exit;
	}

	/**
	 * admin_dashboard function.
	 *
	 * @access public
	 * @return void
	 */
	public function admin_dashboard() {
		new DLM_Admin_Dashboard();
	}
}